function [ data ] = extract_sp(data, attributes, attName, prelInt, ppd, t_min, eta_D, eta_CD, eta_PD, eta_maxFix, eta_minSmp, phi)
%Classifies intervals as fixations or SP
%   @data of gaze recordings
%   attributes describing data
%   attName name of attribute to use
%   preInt preliminary intervals for attName   
%
%   @t_min is the threshold for minimal fixation duration, default is 40ms
%   @eta_D is the threshold for p_D (dispersion), default is 0.45
%   @eta_CD is the threshold for consistency of direction, default is 0.5
%   @eta_PD is the threshold for position displacement, default is 0.2
%   @eta_maxFix is the threshold for spacial range, default is 1.9 deg 
%   @eta_minSmp is the threshold for merged segments spacial range, default is
%   1.7 deg
%   @phi is the threshold for mean direction difference, default is pi/4
%
%   @result is in the same format as @intervals

    DEG_TO_PIXELS = ppd;

    if nargin < 12
        phi = pi / 4;
    end
    if nargin < 11
        eta_minSmp = 1.7;
    end
    if nargin < 10
        eta_maxFix = 1.9;
    end
    if nargin < 9
        eta_PD = 0.2;
    end
    if nargin < 8
        eta_CD = 0.5;
    end
    if nargin < 7
        eta_D = 0.45;
    end
    if nargin < 6
        t_min = 40.;
    end

    % convert minSp,maxFix to pixels
    eta_minSmp = eta_minSmp * DEG_TO_PIXELS;
    eta_maxFix = eta_maxFix * DEG_TO_PIXELS;

    % find position of attributes
    timeIndex = GetAttPositionArff(attributes, 'time');
    xIndex = GetAttPositionArff(attributes, 'x');
    yIndex = GetAttPositionArff(attributes, 'y');
    attIndex = GetAttPositionArff(attributes, attName);
   
    moveId = 0; % unassigned
    index = GetIntervalsIndexArff(data, attributes, attName, moveId);
    segment_class = zeros(size(prelInt,1),1); % 0 is unsure, 1 is SP, -1 if fixation
    segment_parameters = zeros(size(prelInt, 1), 5); % 4 criteria and mean angle
    segment_intersacc_index = zeros(size(prelInt,1),1);
   
    segm_i = 1;
    for i = 1:size(index, 1)
    	start_ind = index(i,1);
   		end_ind = index(i,2);
		
		while segm_i < size(prelInt, 1)    	
	    	segm_begin =  prelInt(segm_i,1);
	    	segm_end = prelInt(segm_i,2);

	    	if segm_begin < start_ind || segm_end > end_ind
	    		break;
            end
            
            if segm_begin == segm_end
                segment_class(segm_i) = -1; % segment of 0 length, let's label it as a fixation
                segm_i = segm_i + 1;
                continue
            end
            if (data(segm_end, timeIndex) - data(segm_begin, timeIndex)) < (t_min * 1000) % milliseconds to microseconds
                segment_class(segm_i) = -1; % too short segment, let it be a fixation
                segm_i = segm_i + 1;
                continue
            elseif (segm_end-segm_begin < 3) %% added by ioannis for jumps in time
                segment_class(segm_i) = -1; % too short segment, let it be a fixation
                segm_i = segm_i + 1;
                continue
            end
            
	    	segment_intersacc_index(segm_i) = i;

	    	part = [data(segm_begin:segm_end, xIndex) data(segm_begin:segm_end, yIndex)]; % XY of points

	    	[coeff, transformed] = pca(part);
	    	d_pc = max(transformed) - min(transformed);
	    	d_ed = norm(part(end, :) - part(1, :));
	    	shifts = diff(part);
	    	traj_len = sum(sqrt(shifts(:, 1).^2 + (shifts(:, 2).^2)));
	    	sp_range = norm(max(part) - min(part));
	    	%mean_angle = mean(atan2(shifts(:, 2), shifts(:, 1)));
	    	mean_angle = circ_mean(atan2(shifts(:, 2), shifts(:, 1)));

	    	p_D = d_pc(2) / d_pc(1);
	    	p_CD = d_ed / d_pc(1);
	    	p_PD = d_ed / traj_len;
	    	p_R = sp_range;

	    	criteria = [-p_D, p_CD, p_PD, p_R] > [-eta_D, eta_CD, eta_PD, eta_maxFix];
	    	
	    	segment_parameters(segm_i, 1:4) = criteria;
	    	segment_parameters(segm_i, 5) = mean_angle;

    		if sum(criteria) == 4
    			segment_class(segm_i) = 1; 
    		elseif sum(criteria) == 0
    			segment_class(segm_i) = -1;
    		else
    			% uncertain segment
    			% do nothing yet
    		end

	    	segm_i = segm_i + 1;
	    end
	end

    for segm_i = 1:size(prelInt, 1)
    	if segment_class(segm_i) == 0
    		% uncertain segment
    		if segment_parameters(segm_i, 3) == 0 %similar to fixation
    			if segment_parameters(segm_i, 4) == 1
    				segment_class(segm_i) = 2; % SP, as determined on 2nd stage
    			else
    				segment_class(segm_i) = -1; % fixation
    			end
    		else % similar to SP
    			% FIXME subject to change if a typo in a paper is found around Table 1
    			assert(eta_maxFix >= eta_minSmp)
    			% then if we find any SP segments withgin the same intersaccadic interval, it's a SP
    			target_ISI = segment_intersacc_index(segm_i);
    			for search_i = 1:size(prelInt, 1)
    				if segment_intersacc_index(search_i) < target_ISI
    					continue
    				elseif  segment_intersacc_index(search_i) > target_ISI
    					break
    				end
    				if segment_class(search_i) ~= 1
    					continue
    				end
    				% angle comparisson
    				%if abs(segment_parameters(search_i, 5) - segment_parameters(segm_i, 5)) <= phi
                    angle1 = segment_parameters(search_i, 5);
                    vec1 = [cos(angle1) sin(angle1)];
                    angle2 = segment_parameters(segm_i, 5);
                    vec2 = [cos(angle2) sin(angle2)];
                    rel_angle = acos(dot(vec1,vec2));
                    if (rel_angle <= phi)
    					segment_class(segm_i) = 2; % SP, as determined on 2nd stage
    				end
    			end
    			if segment_class(segm_i) == 0
    				segment_class(segm_i) = -1; % if no similar SP were found, it's a fixation
    			end
    		end
    	end

    	if segment_class(segm_i) > 0 %any of SP
		    data(prelInt(segm_i,1):prelInt(segm_i,2), attIndex) = 3; % assign sp
    	end

        if segment_class(segm_i) < 0 % -1 fixation
		    data(prelInt(segm_i,1):prelInt(segm_i,2), attIndex) = 1; % assign fixation
        end
    end
end
