function [ dataOut ] = preprocessing(data, attributes, attName, thd, ppd)
%This function processes the intersaccadic intervals (for 1 observer)
%
%   @data from ARFF
%   @attributes describing data
%   @attName name of the attribute to consider for intervals
%   @thd denotes the velocity threshold (in deg/s, samples with speed above which from the 
%   beginning and end of each interval will be removed). Default value is 100
%   deg/s.
%   @ppd pixels per degree for our data
%
%   @dataOut intersaccadic interval coordinates after high velocity changes removal
%   @indexOut new index for dataOut

if nargin < 5
    ppd = 26;
end
if nargin < 4
    thd = 100.;
end 

%intArray = [index size(data,1)+1];
moveId = 0; % unassigned part
intArray = GetIntervalsIndexArff(data, attributes, attName, moveId);
% get position for time, x, y
timeIndex = GetAttPositionArff(attributes, 'time');
xIndex = GetAttPositionArff(attributes, 'x');
yIndex = GetAttPositionArff(attributes, 'y');
attIndex = GetAttPositionArff(attributes, attName);


exceedVel = zeros(1, size(data,1)); % denotes if a pair exceeds velocity threshold
thdPixel = thd*ppd;

% fill exceedVel array
for i=1:size(intArray,1)
    startIndex = intArray(i,1);
    endIndex = intArray(i,2);

    exceedVel(endIndex) = 1; % last element always 1. If previous is 0 the last  will be added
    for j=startIndex:endIndex-1
        disp = sqrt((data(j,xIndex)-data(j+1,xIndex))^2 + (data(j,yIndex)-data(j+1,yIndex))^2);
        dt = (data(j+1,timeIndex)-data(j,timeIndex))/1000000; % in seconds
        disp;
        thdPixel*dt;
        if (disp > thdPixel*dt)
           exceedVel(j) = 1;
        end
    end
end

zeroOneTransitions = 0; % amount of valid transition from 0->1. This kind of one is added to the out
% leave assignement only at start-end of each intersaccadic interval
for i=1:size(intArray,1)
    startIndex = intArray(i,1);
    endIndex = intArray(i,2);
    PSOstart = endIndex; % just to initialize at something that doesn't break the last if
    PSOend = startIndex;

    for j=startIndex:endIndex
        if (exceedVel(j) == 0)
            PSOstart = j;
            break;
        end
    end

    for j=endIndex:-1:startIndex
        if (exceedVel(j) == 0)
            PSOend = j;
            break;
        end
    end
    
    if (PSOstart < PSOend)
        exceedVel(PSOstart:PSOend) = 0;
        zeroOneTransitions = 1 + zeroOneTransitions; % since last element is always 1 and we have at least one zero before
    end
end

% assign data to dataOut 
dataOut = data;

% where velocity exceeds threshold assign value of 4 (noise)
dataOut(exceedVel(:)==1,attIndex) = 4;
end
