% function DetectBlinks:
% this function detects blinks by using intervals of noise in arff as well as saccade detection. For
% every noise interval searches on both sides of it for saccades. Those saccades are marked
% as noise as well
%
% input:
%   inputfile   - arff file containing gaze coordinates
%   paramfile   - (optional) txt file containing parameters for blinks detection. Same as saccade detection 
%
% output:
%   result      - column vector with same length as data and value of 4 for every sample that is part of a blink

function result = DetectBlinks(inputfile, paramfile)
	% initialize search interval on both sides of the blink in us
	c_searchRange = 30000;

    % load confidence interval in coord format (0=noise)
    [gaze, pixelX, pixelY, width, height, distance, conf] = LoadCoordAndConfArff(inputfile);

    % convert to arff format (4=noise, 0=unassigned)
    noise = zeros(size(conf));
    noise(conf(:) < 0.25) = 4;

    % detect saccades
    if (nargin < 2) %use default
        saccades = DetectSaccadesDSF(inputfile);
    else
        saccades = DetectSaccadesDSF(inputfile, paramfile);
    end

    % initially
    result = noise;

    % search for noise in noiseidence
    isNoiseActive = 0;
    startIndex = -1;
    endIndex = -1;
    for noiseIndex=1:size(noise,1)
        if (isNoiseActive == 0 && noise(noiseIndex) == 4)
            isNoiseActive = 1;
            startIndex = noiseIndex;
        end

        if (isNoiseActive == 1 && noise(noiseIndex) == 0)
            isNoiseActive = 0;
            endIndex = noiseIndex-1;
            UpdateResult();
        end
    end

    % function UpdateResult:
    % in seraches on both sides of the noise interval and searches for blinks

    function UpdateResult()
        % search backwards
        searchIndex = startIndex;
        saccadeFound = false;
        while (searchIndex > 0)
            if (gaze(startIndex,1)-gaze(searchIndex,1) > c_searchRange && saccadeFound==false)
                break;
            end

            if (saccades(searchIndex)==2 && saccadeFound==false)
                saccadeFound = true;
            end

            if (saccades(searchIndex)~=2 && saccadeFound==true)
                result(searchIndex+1:startIndex) = 4;
                break;
            end
        
            searchIndex = searchIndex-1;
        end

        % search forward
        searchIndex = endIndex;
        saccadeFound = false;
        while (searchIndex <= size(gaze,1))
            if (gaze(searchIndex,1)-gaze(endIndex,1) > c_searchRange && saccadeFound==false)
                break;
            end

            if (saccades(searchIndex)==2 && saccadeFound==false)
                saccadeFound = true;
            end

            if (saccades(searchIndex)~=2 && saccadeFound==true)
                result(endIndex+1:searchIndex) = 4;
                break;
            end
        
            searchIndex = searchIndex+1;
        end

    end

end
