% function DetectSaccadesDSF:
% this function detects saccades from the input file. It tries to replicate SaccadeDetector
% of dsf for use in matlab. So for comparison of the logic look this documentation
%
% input:
%   inputfile   - arff file containing gaze coordinates
%   paramfile   - (optional) txt file containing parameters for saccade detection (explanation below)
%
% output:
%   result      - column vector with the same length as data in inputfile and value 2 where a saccade is detected
%
% paramfile format:
% The file is indipendent of parameter ordering and letter case. Each parameter is followed by 
% an equal sign and then the value. The available values are below
%   tolerance=
%   thresholdOnsetFast=
%   thresholdOnsetSlow=
%   thresholdOfset=
%   maxSpeed=
%   minDuration=
%   maxDuration=
%   velIntegrationInterv=
%
% The meaning of each argument is copy pasted from dsf below
%{
	double     tolerance;
       ///< The relative size of the area outside the screen 
       ///< that is still considered to be legal

   double     thresholdOnsetFast;
       ///< Threshold for initialization of saccade detection
       ///< This and the following thresholds are in deg/s.

   double     thresholdOnsetSlow;
       ///< Once velocity is above thresholdOnsetFast,
       ///< the first sample where velocity was > thresholdOnsetSlow
       ///< is marked as saccade onset.

   double     thresholdOffset;
       ///< Maximum velocity that is regarded as end of saccade.

   double     maxSpeed;
       ///< Maximum speed of saccadic eye movements. Velocities
       ///< exceeding this value are treated as impulse noise.

   int64_t   minDuration;
       ///< Saccades of a duration less than this (in microseconds)
       ///< are treated as impulse noise.

   int64_t   maxDuration;
       ///< Max duration that the saccade status is held upright.
       ///< During a glitch, an offset might not be detected,
       ///< so eventually the saccade flag is switched off anyway.
       ///< This value is given in microseconds. The current default
       ///< of 160000us roughly equals an amplitude of >50 degrees.

   int64_t   velIntegrationInterv;
       ///< Interval over which to integrate velocity computation.
       ///< For very high sampling rates (i.e., SMI's 1250 Hz),
       ///< calculating sample-to-sample velocity is not feasible.
       ///< The current default of 2000us means that for 240 Hz
       ///< (SMI Hi-Speed) or 250 Hz (EyeLink) recordings,
       ///< only one sample is taken into account as before; for
       ///< 1250 Hz recordings, though, we compute velocity on
       ///< three samples
%}


function result = DetectSaccadesDSF(inputfile, paramfile)
    % initialize detection parameters
    if (nargin < 2)
        c_tolerance = 0.1;
        c_thresholdOnsetFast = 137.5;
        c_thresholdOnsetSlow = 17.1875;
        c_thresholdOffset = 17.1875;
        c_maxSpeed = 1031.25;
        c_minDuration = 15000;
        c_maxDuration = 160000;
        c_velIntegrationInterv = 4000;
    else
        params = LoadParams(paramfile);
        c_tolerance = params.tolerance;
        c_thresholdOnsetFast = params.thresholdonsetfast;
        c_thresholdOnsetSlow = params.thresholdonsetslow;
        c_thresholdOffset = params.thresholdoffset;
        c_maxSpeed = params.maxspeed;
        c_minDuration = params.minduration;
        c_maxDuration = params.maxduration;
        c_velIntegrationInterv = params.velintegrationinterv;
    end

    % load gaze coordinates from arff file
    % LoadCoordAndConf returns confidence in .coord format (0=noise)
    [gaze, pixelX, pixelY, width, height, distance, conf] = LoadCoordAndConfArff(inputfile);
    [ppdx, ppdy] = PixelsPerDegree(width, height, distance, pixelX, pixelY);

    % initialize result
    result = zeros(size(gaze,1),1);

    if (size(gaze,1) < 10)
        return;
    end

    % convert gaze and screen size from pixels to degrees
    gaze(:,2) = gaze(:,2)./ppdx;
    gaze(:,3) = gaze(:,3)./ppdy;
    pixelX = pixelX/ppdx;
    pixelY = pixelY/ppdy;


    % calculate velocity 
    velocity = zeros(size(gaze,1),1);
    % *** the integration interval is not implemented yet
    % get amplitude per sample
    %{ 
    velocity(1:end-1) = sqrt( (gaze(1:end-1,2)-gaze(2:end,2)).^2 + (gaze(1:end-1,3)-gaze(2:end,3)).^2 );
    % devide by time
    velocity(1:end-1) = 1000000*velocity(1:end-1)./(gaze(2:end,1)-gaze(1:end-1,1)); % convert to deg/s
    velocity(end) = velocity(end-1);
    %}
    
    velocity(2:end) = sqrt( (gaze(1:end-1,2)-gaze(2:end,2)).^2 + (gaze(1:end-1,3)-gaze(2:end,3)).^2 );
    % devide by time
    velocity(2:end) = 1000000*velocity(2:end)./(gaze(2:end,1)-gaze(1:end-1,1)); % convert to deg/s
    velocity(1) = velocity(2);
    
   
    %{
    % velocity integration
    for i=1:size(velocity,1)-1
        j = i+1;
        while (j < size(velocity,1) && gaze(j,1)-gaze(i,1) < c_velIntegrationInterv)
            j = j+1;
        end
        velocity(i) = sqrt( (gaze(i,2)-gaze(j,2)).^2 + (gaze(i,3)-gaze(j,3)).^2 );
        velocity(i) = 1000000*velocity(i)./(gaze(j,1)-gaze(i,1)); % convert to deg/s
    end
    %}
    

    % create glitch array
    glitch = zeros(size(gaze,1),1);
    glitch(gaze(:,2) > (1+c_tolerance)*pixelX) = 1;
    glitch(gaze(:,2) < -c_tolerance*pixelX) = 1;
    glitch(gaze(:,3) > (1+c_tolerance)*pixelY) = 1;
    glitch(gaze(:,3) < -c_tolerance*pixelY) = 1;
    glitch(conf(:) < 0.25) = 1;

    isSaccActive = 0;
    onsetSlowIndex = 1;

    for i=1:size(gaze,1)
        % not in glitch
        if (glitch(i) == 0)
            if (isSaccActive == 0)
                % if speed less than onset slow move index
                if (velocity(i) < c_thresholdOnsetSlow)
                    onsetSlowIndex = i+1;
                end

                % saccade above fast threshold but below physically impossible
                if (velocity(i) > c_thresholdOnsetFast && velocity(i) < c_maxSpeed)
                    isSaccActive = 1;
                    
                    % allocate all samples from onset slow as saccade
                    result(onsetSlowIndex:i) = 2;
                end
            end

            % if within saccade check for termination cases otherwise make sample part of saccade
            if (isSaccActive == 1)
                result(i) = 2;
                % saccade termination cases
                if (velocity(i) < c_thresholdOffset)
                    isSaccActive = 0;

                    % check for minDuration
                    if (gaze(i,1)-gaze(onsetSlowIndex,1) < c_minDuration)
                        result(onsetSlowIndex:i) = 0;
                    end
                    continue; % skip rest of the loop
                end

                if (gaze(i,1)-gaze(onsetSlowIndex,1) > c_maxDuration)
                    isSaccActive = 0;
                    continue;
                end

                % check if onset and current point are the same
                if (i-onsetSlowIndex < 2)
                    continue
                end
                meanVel = sqrt( (gaze(i,2)-gaze(onsetSlowIndex,2))^2 + (gaze(i,3)-gaze(onsetSlowIndex,3))^2 );
                meanVel = 1000000*meanVel/(gaze(i,1)-gaze(onsetSlowIndex,1)+1); % +1 avoid division by 0
                if (meanVel < c_thresholdOnsetSlow)
                    isSaccActive = 0;

                    % check for minDuration
                    if (gaze(i,1)-gaze(onsetSlowIndex,1) < c_minDuration)
                        result(onsetSlowIndex:i) = 0;
                    end
                    continue;
                end

            end
        else
            onsetSlowIndex = i+1;
            result(i) = 4;
        end
    end
end
