% function LoadArff:
% loads an ARFF and returns the data, metadata, attributes and relation. All returned 
% strings are lower case
%
% input:
%   arffFile    - path to arff file to read
%   
% output:
%   data        - data stored in arff file
%   metadata    - metadata part of the header content metadata.{pixelX,pixelY,dimensionX,dimensionY,distance} -1 if not available. metadata.extrnx2 cell array for every other pair of name-value metadata
%   attributes  - nx1 cell array with attribute names, where n is the number of attributes
%   relation    - relation described in arff

function [data, metadata, attributes, relation] = LoadArff(arffFile)
    % initialize data
    data = [];
    % initial metadat
    metadata.pixelX = -1;
    metadata.pixelY = -1;
    metadata.dimensionX = -1; % meters
    metadata.dimensionY = -1; % meters
    metadata.distance = -1; % meters
    % stable metadata
    metadata.width_px = -1;
    metadata.height_px = -1;
    metadata.width_mm = -1; 
    metadata.height_mm = -1; 
    metadata.distance_mm = -1; 
    metadata.extra = {};
    attributes = {};
    relation = '';

    % read header
    numOfHeaderLines = 1;
    fid = fopen(arffFile, 'r');
    fline = fgetl(fid);
    while (ischar(fline))
        % split lines into words
        words = strsplit(fline,' ');
        % check for relation
        if (size(words,2)>1 && strcmpi(words{1,1},'@relation')==1)
            relation = lower(words{1,2});
        % check for pixelX or width_px
        elseif (size(words,2)>2 && strcmpi(words{1,1},'%@metadata')==1 && (strcmpi(words{1,2},'pixelX')==1 || strcmpi(words{1,2},'width_px')==1))
            metadata.pixelX = str2num(words{1,3});
            metadata.width_px = metadata.pixelX;
        % check for pixelY
        elseif (size(words,2)>2 && strcmpi(words{1,1},'%@metadata')==1 && (strcmpi(words{1,2},'pixelY')==1 || strcmpi(words{1,2},'height_px')==1))
            metadata.pixelY = str2num(words{1,3});
            metadata.height_px = metadata.pixelY;
        % check for dimensionX
        elseif (size(words,2)>2 && strcmpi(words{1,1},'%@metadata')==1 && strcmpi(words{1,2},'dimensionX')==1)
            metadata.dimensionX = str2num(words{1,3});
            metadata.width_mm = metadata.dimensionX*1000;
        % check for dimensionY
        elseif (size(words,2)>2 && strcmpi(words{1,1},'%@metadata')==1 && strcmpi(words{1,2},'dimensionY')==1)
            metadata.dimensionY = str2num(words{1,3});
            metadata.height_mm = metadata.dimensionY*1000;
        % check for distance
        elseif (size(words,2)>2 && strcmpi(words{1,1},'%@metadata')==1 && strcmpi(words{1,2},'distance')==1)
            metadata.distance = str2num(words{1,3});
            metadata.distance_mm = metadata.distance*1000;
        % check for width_mm
        elseif (size(words,2)>2 && strcmpi(words{1,1},'%@metadata')==1 && strcmpi(words{1,2},'width_mm')==1)
            metadata.width_mm = str2num(words{1,3});
            metadata.dimensionX = metadata.width_mm/1000;
        % check for height_mm
        elseif (size(words,2)>2 && strcmpi(words{1,1},'%@metadata')==1 && strcmpi(words{1,2},'height_mm')==1)
            metadata.height_mm = str2num(words{1,3});
            metadata.dimensionY = metadata.height_mm/1000;
        % check for distance_mm
        elseif (size(words,2)>2 && strcmpi(words{1,1},'%@metadata')==1 && strcmpi(words{1,2},'distance_mm')==1)
            metadata.distance_mm = str2num(words{1,3});
            metadata.distance = metadata.distance_mm/1000;
        % process the rest of the metadata
        elseif (size(words,2)>2 && strcmpi(words{1,1},'%@metadata')==1)
        metadata.extra{end+1,1} = {words{1,2}, words{1,3}};
        % check for attributes
        elseif (size(words,2)>2 && strcmpi(words{1,1},'@attribute')==1)
            index = size(attributes,1)+1;
            attributes{index,1} = lower(words{1,2});
            attributes{index,2} = words{1,3};
        % check if data has been reached
        elseif (size(words,2)>0 && strcmpi(words{1,1},'@data')==1)
            break;
        end

        fline = fgetl(fid);
        numOfHeaderLines = numOfHeaderLines+1;
    end    
    fclose(fid);

    % load data
    dataCell = importdata(arffFile,',',numOfHeaderLines);
    if (isfield(dataCell,'data'))
        data = dataCell.data;
    else
        data = zeros(0, size(attributes,1));
    end
end    
