% function SaveArff:
% save provided in an ARFF formated file. Only numeric attributes are
% accepted.
% This function is suitable for the 'gaze_recording' @RELATIONs, where the
% metadata describes the setup of the eyetracking experiment:
%   - width_px/height_px/width_mm/height_mm dewscribe the pixels-size and
%     the physical size of the video surface
%   - distance_mm describes how far were the observer's eyes from the
%     monitor
% The description of the file format (esp. the 'larsson' column) is printed 
% in the beginning of the resulting .arff file.
% 
% input:
%   arffFile    - name of the file to save data
%   data        - data to write in arff file
%   metadata    - metadata struct in the form of metadata.{pixelX,pixelY,dimensionY,dimensionY,distance}
%   attributes  - nx1 cell array holding the attribute names
%   relation    - relation dscribed inthe file

function SaveArff(arffFile, data, metadata, attributes, relation)
    % check input
    assert(isfield(metadata,'width_px'), 'metadata should contain "width_px" field');
    assert(isfield(metadata,'height_px'), 'metadata should contain "height_px" field');
    assert(isfield(metadata,'width_mm'), 'metadata should contain "width_mm" field');
    assert(isfield(metadata,'height_mm'), 'metadata should contain "height_mm" field');
    assert(isfield(metadata,'distance_mm'), 'metadata should contain "distance_mm" field');
    assert(size(relation,2)>0, 'relation should not be empty');
    assert(size(attributes,1)==size(data,2), 'attribute number should be the same with data');

    % start writing
    fid = fopen(arffFile, 'w+');
    
    % write format description
    fprintf(fid, ['%% The column "larsson" contains the eye movement labels produced by our re-implementation of the Larsson fixation and SP detection algorithm.\n', ...
                  '%% The labels in this column are to be interpreted as follows:\n', ...
                  '%%   - 0 is UNKNOWN\n', ...
                  '%%   - 1 is FIX (fixation)\n', ...
                  '%%   - 2 is SACCADE\n', ...
                  '%%   - 3 is SP (smooth pursuit)\n', ...
                  '%%   - 4 is NOISE\n']);

    % write relation
    fprintf(fid, '@RELATION %s\n\n', relation);

    % write metadata
    fprintf(fid, '%%@METADATA width_px %d\n', metadata.width_px);
    fprintf(fid, '%%@METADATA height_px %d\n', metadata.height_px);
    fprintf(fid, '%%@METADATA width_mm %.2f\n', metadata.width_mm);
    fprintf(fid, '%%@METADATA height_mm %.2f\n', metadata.height_mm);
    fprintf(fid, '%%@METADATA distance_mm %.2f\n\n', metadata.distance_mm);

    % write metadata extras. Those are data that vary between experiments
    for i=1:size(metadata.extra,1)
        fprintf(fid, '%%@METADATA %s %s\n', metadata.extra{i,1}{1,1}, metadata.extra{i,1}{1,2});
    end
    % print an empty line
    fprintf(fid, '\n');

    % write attributes
    for i=1:size(attributes,1)
        fprintf(fid, '@ATTRIBUTE %s %s\n', attributes{i,1}, upper(attributes{i,2}));
    end

    % write data keyword
    fprintf(fid,'\n@DATA\n');

    % write actual data
    for i=1:size(data,1)
        for j=1:size(data,2)
            if (strcmpi(attributes{j,2},'integer')==1) 
                fprintf(fid,'%d',data(i,j));
            elseif (strcmpi(attributes{j,2},'numeric')==1)
                fprintf(fid,'%.2f',data(i,j));
            else
                error(['Attribute type "' attributes{j,2} '" is not recognised']);
            end

            if (j<size(data,2))
                fprintf(fid,',');
            end
        end
        fprintf(fid,'\n');
    end

    % close file
    fclose(fid);
end    
